
#include "../include/ARPluginHelper.h"
#include "eq.h"

#define EFFECT_ID 4458
#define EFFECT_VERSION 1

#define EFFECT_EDITOR_WIDTH 200
#define EFFECT_EDITOR_HEIGHT 130

#define LOW_FREQ 500
#define HIGH_FREQ 5000

struct PluginState
{
	float bassGain;
	float midGain;
	float trebGain;
};

class EQEditor : public CRUnknown, public ARSliderListener
{
protected:
	ARLabel* bassLabel;
	ARLabel* midLabel;
	ARLabel* trebLabel;

	ARSlider* bassSlider;
	ARSlider* midSlider;
	ARSlider* trebSlider;

public:
	EQSTATE eqStateLeft;
	EQSTATE eqStateRight;

	PluginState pluginState;

	EQEditor()
	{
		ZeroMemory(&eqStateLeft, sizeof(EQSTATE));
		ZeroMemory(&eqStateRight, sizeof(EQSTATE));

		// default gain values
		pluginState.bassGain = 1.0f;
		pluginState.midGain = 1.0f;
		pluginState.trebGain = 1.0f;

		bassLabel = 0;
		midLabel = 0;
		trebLabel = 0;

		bassSlider = 0;
		midSlider = 0;
		trebSlider = 0;
	}

	void UpdateSliderValues()
	{
		if (bassSlider) // update only if created
		{
			bassSlider->SetValue((int)((pluginState.bassGain / 2.0f) * 100));
			midSlider->SetValue((int)((pluginState.midGain / 2.0f) * 100));
			trebSlider->SetValue((int)((pluginState.trebGain / 2.0f) * 100));
		}
	}

	void UpdateEqGainValues()
	{
		eqStateLeft.lg = pluginState.bassGain;
		eqStateLeft.mg = pluginState.midGain;
		eqStateLeft.hg = pluginState.trebGain;

		eqStateRight.lg = pluginState.bassGain;
		eqStateRight.mg = pluginState.midGain;
		eqStateRight.hg = pluginState.trebGain;
	}

	void CreateEditor(HWND parentWindow, UserInterfaceAPI* interfaceAPI)
	{
		interfaceAPI->CreateSlider(parentWindow, true, &bassSlider);
		interfaceAPI->CreateSlider(parentWindow, true, &midSlider);
		interfaceAPI->CreateSlider(parentWindow, true, &trebSlider);

		interfaceAPI->CreateLabel(parentWindow, false, &bassLabel);
		interfaceAPI->CreateLabel(parentWindow, false, &midLabel);
		interfaceAPI->CreateLabel(parentWindow, false, &trebLabel);

		bassSlider->SetPosition(60, 20);
		midSlider->SetPosition(90, 20);
		trebSlider->SetPosition(120, 20);

		bassLabel->SetPosition(55, 105);
		midLabel->SetPosition(88, 105);
		trebLabel->SetPosition(118, 105);

		bassSlider->SetSize(15, 80);
		midSlider->SetSize(15, 80);
		trebSlider->SetSize(15, 80);

		this->UpdateSliderValues();

		bassSlider->SetListener(this);
		midSlider->SetListener(this);
		trebSlider->SetListener(this);

		bassLabel->SetText(RSTR(L"bass"));
		midLabel->SetText(RSTR(L"mid"));
		trebLabel->SetText(RSTR(L"treb"));

		bassLabel->SetTextColor(RGB(200, 100, 100));
		midLabel->SetTextColor(RGB(100, 200, 200));
		trebLabel->SetTextColor(RGB(100, 200, 100));
	}

	void STDCALL OnSliderChange(int sliderID)
	{
		if (sliderID == bassSlider->GetID())
			pluginState.bassGain = (bassSlider->GetValue() / 100.0f) * 2;
		else if (sliderID == midSlider->GetID())
			pluginState.midGain = (midSlider->GetValue() / 100.0f) * 2;
		else if (sliderID == trebSlider->GetID())
			pluginState.trebGain = (trebSlider->GetValue() / 100.0f) * 2;

		UpdateEqGainValues();
	}

	void DestroyEditor()
	{
		bassSlider->Release();
		midSlider->Release();
		trebSlider->Release();

		bassLabel->Release();
		midLabel->Release();
		trebLabel->Release();

		bassLabel = 0;
		midLabel = 0;
		trebLabel = 0;

		bassSlider = 0;
		midSlider = 0;
		trebSlider = 0;
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

class EQPlugin : public ARPluginHelper
{
protected:
	EQEditor* editor;
	
public:
	EQPlugin()
	{
		editor = new EQEditor();
	}

	virtual ~EQPlugin()
	{
		editor->Release();
	}

	void STDCALL SetSampleRate(float srate)
	{
		init_3band_state(&editor->eqStateLeft, LOW_FREQ, HIGH_FREQ, (int)srate);
		init_3band_state(&editor->eqStateRight, LOW_FREQ, HIGH_FREQ, (int)srate);

		// re-set gain values
		editor->UpdateEqGainValues();
	}

	void STDCALL ProcessAudio(float **inputs, float **outputs, long frameCount)
	{
		for (int i = 0; i < frameCount; ++i)
		{
			outputs[0][i] = (float)do_3band(&editor->eqStateLeft, inputs[0][i]);
			outputs[1][i] = (float)do_3band(&editor->eqStateRight, inputs[1][i]);
		}
	}

	void STDCALL GetEffectName(RString** ppText)
	{
		*ppText = MakeRString(L"3 Band EQ");
	}

	int STDCALL GetEffectID()
	{
		return EFFECT_ID;
	}

	int STDCALL GetEffectVersion()
	{
		return EFFECT_VERSION;
	}

	void STDCALL GetEditorSize(int *width, int *height)
	{
		*width = EFFECT_EDITOR_WIDTH;
		*height = EFFECT_EDITOR_HEIGHT;
	}

	void STDCALL CreateEditor(HWND parentWindow)
	{
		editor->CreateEditor(parentWindow, interfaceAPI);
	}

	void STDCALL DestroyEditor()
	{
		editor->DestroyEditor();
	}

	void STDCALL LoadState(void *data, int size)
	{
		if (size == sizeof(PluginState))
		{
			editor->pluginState = *(PluginState*)data;
			editor->UpdateEqGainValues();
			editor->UpdateSliderValues();
		}
	}

	int STDCALL SaveState(void **data)
	{
		*data = &editor->pluginState;
		return sizeof(PluginState);
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_EFFECT(EQPlugin)