
// Interfaces for general plugins.
// (c) 2021 CrownSoft

#ifndef _AR_GEN_INTERFACES_H_
#define _AR_GEN_INTERFACES_H_

#include "RPlugin.h"

#ifndef AUDIO_REPEATER_PRO_HOST_ID
	#define AUDIO_REPEATER_PRO_HOST_ID 9980
#endif

#define AUDIO_REPEATER_PRO_SYSTEM_API_V1 2001
#define AUDIO_REPEATER_PRO_SYSTEM_API_V2 2002

class ARSystemAPIFactory : public RUnknown
{
public:
	virtual int STDCALL GetSystemAPI(int apiID, void** pObject) = 0;
};

class ARServiceFactory : public RUnknown
{
public:
	virtual void STDCALL GetFactoryName(RString** ppText) = 0;
	virtual int STDCALL GetService(int serviceID, void** pObject) = 0;
};

class ARAudioCallback : public RUnknown
{
public:
	// called by the audio thread.
	// Since ARAudioCallback is only active in processing mode, channel count is always 2.
	// the interval between each call is not constant.
	// outputBuffer contains processed audio data by the effect plugins. you can modify this buffer if you want to apply any effects.
	// inputBuffer contains unmodified audio data captured from the input device.
	virtual void STDCALL OnAudioData(float* inputBuffer, float* outputBuffer, int framesCount, unsigned int byteCount, unsigned int sampleRate) = 0;
};

class ARMenuItemListener : public RUnknown
{
public:
	virtual void STDCALL OnMenuPress(int itemID) = 0;
};

class ARThread : public RUnknown
{
public:
	virtual void STDCALL ThreadProc(int threadID) = 0;
};

// ============= Events generated by the Audio Repeater =================

/*
* AR_EVENT_DEVICE_STATE_CHANGE:
*	Generated when audio device state changed.
*	param1 will be AR_DEVICE_STATE_XXX constants.
*	param2 will be the device id. (const wchar_t*)
*/
#define AR_EVENT_DEVICE_STATE_CHANGE	4000

#define AR_EVENT_DEVICE_LIST_RELOADED	4001
#define AR_EVENT_STREAM_START			4002
#define AR_EVENT_STREAM_STOP			4003

// used to send custom events.
// can be used when you need to do some work on gui thread.
// ex:	#define MY_EVENT1	AR_CUSTOM_EVENT + 1
//		#define MY_EVENT2	AR_CUSTOM_EVENT + 2
#define AR_CUSTOM_EVENT					20000


// AR_DEVICE_STATE_XXX constants which comes as a param1 of AR_EVENT_DEVICE_STATE_CHANGE event. (There can be undocumented values.)
#define AR_DEVICE_STATE_ACTIVE      0x00000001
#define AR_DEVICE_STATE_DISABLED    0x00000002
#define AR_DEVICE_STATE_NOTPRESENT  0x00000004
#define AR_DEVICE_STATE_UNPLUGGED   0x00000008

// ======================================================================

class AREventListener : public RUnknown
{
public:
	virtual void STDCALL OnAREvent(int eventID, int param1, void* param2) = 0;
};

namespace ARMsgIconTypes {
	enum ARMsgIconTypes
	{
		MSG_INFORMATION = 79,
		MSG_ERROR = 80,
		MSG_QUESTION = 23,
		MSG_WARNING = 78
	};
}

namespace ARMsgButtonTypes {
	enum ARMsgButtonTypes
	{
		MSG_BUTTON_OK = 0,
		MSG_BUTTON_YES_NO = 1
	};
}

class ARSystemAPI : public RUnknown
{
public:

	// ============= AUDIO_REPEATER_PRO_SYSTEM_API_V1 =============

	virtual int STDCALL GetBuildID() = 0;

	virtual int STDCALL GetInputDeviceCount() = 0;
	virtual int STDCALL GetOutputDeviceCount() = 0;

	virtual int STDCALL GetInputDeviceNameByIndex(int index, RString** deviceName) = 0;
	virtual int STDCALL GetInputDeviceIDByIndex(int index, RString** deviceID) = 0;
	virtual int STDCALL GetOutputDeviceNameByIndex(int index, RString** deviceName) = 0;
	virtual int STDCALL GetOutputDeviceIDByIndex(int index, RString** deviceID) = 0;

	virtual void STDCALL GetCurrentInputDeviceID(RString** deviceID) = 0;
	virtual void STDCALL GetCurrentInputDeviceName(RString** deviceName) = 0;
	virtual void STDCALL GetCurrentOutputDeviceID(RString** deviceID) = 0;
	virtual void STDCALL GetCurrentOutputDeviceName(RString** deviceName) = 0;

	virtual int STDCALL SetCurrentInputDevice(RString* deviceID) = 0;
	virtual int STDCALL SetCurrentOutputDevice(RString* deviceID) = 0;

	// device must exist
	virtual int STDCALL GetDeviceNameByID(RString* deviceID, RString** deviceName) = 0;

	virtual int STDCALL IsDeviceAvailable(RString* deviceID) = 0;

	virtual int STDCALL IsStreaming() = 0;
	virtual int STDCALL IsApplicationMinimized() = 0;
	virtual void STDCALL GetApplicationWindow(HWND* windowHandle) = 0;
	virtual void STDCALL MinimizeApplication() = 0; // async call
	virtual void STDCALL RestoreApplication() = 0; // async call

	virtual int STDCALL ShowMessage(int icon, RString* title, RString* text, int buttons) = 0;
	virtual void STDCALL ShowNotification(RString* title, RString* text) = 0;

	// call within RPlugin->Enable(). no multiple entries to the plugin menu. must call RemoveFromPluginMenu on RPlugin->Disable().
	virtual void STDCALL AddToPluginMenu(RString* menuText, ARMenuItemListener* listener) = 0; 

	virtual void STDCALL RemoveFromPluginMenu(ARMenuItemListener* listener) = 0;  // call within RPlugin->Disable()

	virtual int STDCALL GetServiceFactory(RGUID* factoryID, ARServiceFactory** ppFactory) = 0; // cannot be called within RPlugin->Initialize() and RPlugin->Free()
	virtual void STDCALL RegisterServiceFactory(RGUID* factoryID, ARServiceFactory* pFactory) = 0; // call within RPlugin->Initialize(). must call UnregisterServiceFactory on RPlugin->Free()
	virtual void STDCALL UnregisterServiceFactory(RGUID* factoryID) = 0; // call within RPlugin->Free()

	virtual void STDCALL AddAudioCallback(ARAudioCallback* audioCallback) = 0; // call within RPlugin->Enable()
	virtual void STDCALL RemoveAudioCallback(ARAudioCallback* audioCallback) = 0; // call within RPlugin->Disable()

	virtual void STDCALL AddEventListener(AREventListener* eventListener) = 0; // call within RPlugin->Enable()
	virtual void STDCALL RemoveEventListener(AREventListener* eventListener) = 0; // call within RPlugin->Disable()

	// ===== async calls =======
	virtual void STDCALL ReloadDeviceList() = 0;
	virtual void STDCALL StartStreamingByID(RString* inputDeviceID, RString* outputDeviceID, int silentMode) = 0; // also reloads the device list
	virtual void STDCALL StartStreamingByName(RString* inputDeviceName, RString* outputDeviceName, int silentMode) = 0; // also reloads the device list
	virtual void STDCALL StopStreaming() = 0;
	virtual void STDCALL PostEvent(AREventListener* eventListener, int eventID, int param1, void* param2) = 0; // can be used to do some work on gui thread. event may not receive if the app is closing.

	// =================== AUDIO_REPEATER_PRO_SYSTEM_API_V2 =================

	virtual int STDCALL GetCurrentInputDeviceIndex() = 0;
	virtual int STDCALL GetCurrentOutputDeviceIndex() = 0;
	virtual int STDCALL GetCurrentSampleRate() = 0;
	virtual int STDCALL IsProcessingFormat() = 0;
	virtual int STDCALL IsProcessAudioEnabled() = 0;

	// generates a file path in Music dir with the given extension.
	virtual void STDCALL GenerateAudioFilePath(RString* fileExtension, RString** filePath) = 0;

	virtual void STDCALL AllocateMemory(int size, void** buffer) = 0;
	virtual void STDCALL FreeMemory(void* buffer) = 0;

	// must free the utf8Text using FreeMemory.
	virtual void STDCALL ConvertUTF16ToUTF8(const wchar_t* utf16Text, char** utf8Text) = 0;

	// must free the utf16Text using FreeMemory.
	virtual void STDCALL ConvertUTF8ToUTF16(const char* utf8Text, wchar_t** utf16Text) = 0;

	// creates COM initialized thread. User specified threadID can be used to identify each thread at ARThread->ThreadProc method.
	virtual void STDCALL CreateThread(int threadID, HANDLE* threadHandle, ARThread* threadInstance) = 0;

};

#endif