
#pragma once

#include "ARGeneralPlugin.h"
#include "ARStdServices.h"

// Assist you with the windowing and plugin state management. Also hides the COM stuff!
// Plugin window will be automatically created/destroyed for you. :-)
template<class PluginStateStruct>
class ARGeneralPluginHelper : public ARGeneralPlugin, public TWindowListener, public ARMenuItemListener
{
protected:
	PluginStateStruct pluginState;
	bool stateAssigned;

	TWindow* pluginWindow;

public:
	ARGeneralPluginHelper()
	{
		stateAssigned = false;

		pluginState.posX = 0;
		pluginState.posY = 0;
		pluginState.visible = true;

		pluginWindow = NULL;
	}

	virtual ~ARGeneralPluginHelper()
	{

	}

	virtual void STDCALL GetName(RString** ppText) override
	{
		*ppText = MakeRString(this->GetPluginName());
	}

	virtual void STDCALL GetAuthor(RString** ppText) override
	{
		*ppText = MakeRString(this->GetPluginAuthor());
	}

	virtual int STDCALL Initialize(HINSTANCE hInstance, int hostID, void* hostParam) override
	{
		if (ARGeneralPlugin::Initialize(hInstance, hostID, hostParam))
			return this->OnInitialize(hostID, hostParam);
		return 0;
	}

	virtual void STDCALL Enable() override
	{
		if (this->ShouldCreateUI())
			this->CreateUI();

		RString* pluginMenuText = MakeRString(this->GetPluginMenuText());
		systemAPI->AddToPluginMenu(pluginMenuText, this);
		pluginMenuText->Release();

		this->OnEnable();
	}

	virtual void STDCALL Disable() override
	{
		this->OnDisable();

		systemAPI->RemoveFromPluginMenu(this);

		if (pluginWindow)
			this->DestroyUI();

		stateAssigned = true;
	}

	virtual void STDCALL LoadState(void* data, int size) override
	{
		if (size == sizeof(PluginStateStruct))
		{
			pluginState = *(PluginStateStruct*)data;
			stateAssigned = true;

			this->OnStateLoad();

			if (pluginWindow)
				this->ApplyPluginStateToUI();
		}
	}

	virtual int STDCALL SaveState(void** data) override
	{
		this->OnStateSave();

		*data = &pluginState;
		return sizeof(PluginStateStruct);
	}

	virtual void STDCALL Free(int initOK) override
	{
		if (!initOK)
			return;

		this->OnFree();
	}

	virtual void STDCALL OnMenuPress(int itemID) override
	{
		pluginState.visible = true;

		if (pluginWindow == NULL)
			this->CreateUI();
	}

	virtual void STDCALL OnWindowMessage(TWindow* window, UINT message, WPARAM wParam, LPARAM lParam) override
	{
		if (window == pluginWindow)
		{
			if (message == WM_CLOSE)
			{
				pluginState.visible = false;		
				this->DestroyUI();
			}
		}
	}

	virtual void ApplyPluginStateToUI()
	{
		if (stateAssigned) // otherwise centerd
		{
			pluginWindow->SetPosition(pluginState.posX, pluginState.posY);
		}
		else
		{
			// assign centered position to state
			pluginState.posX = pluginWindow->GetPosX();
			pluginState.posY = pluginWindow->GetPosY();
		}

		this->OnApplyPluginStateToUI();

		pluginWindow->SetVisible(1);
	}

	virtual bool ShouldCreateUI()
	{
		return (systemAPI->IsApplicationMinimized() == 1) ? false : pluginState.visible;
	}

	virtual void CreateUI()
	{
		ARServiceFactory* stdServiceFactory;
		if (systemAPI->GetServiceFactory(&AR_STD_SERVICES_FACTORY_ID, &stdServiceFactory))
		{
			ARGUIService* guiService;
			if (stdServiceFactory->GetService(AR_GUI_SERVICE_V2, (void**)&guiService))
			{
				guiService->CreateHostWindow(TWindowTypes::FIXED_SIZE_WINDOW, &pluginWindow);

				RString* windowText = MakeRString(this->GetPluginWindowText());
				pluginWindow->SetText(windowText);
				windowText->Release();

				this->OnCreateUI(stdServiceFactory, guiService);

				pluginWindow->CenterScreen(0);
				pluginWindow->SetListener(this);

				this->ApplyPluginStateToUI();

				guiService->Release();
			}
			else
			{
				MessageBoxW(0, L"Required GUI service not found. Please update your application.", this->GetPluginName(), MB_ICONERROR);
			}

			stdServiceFactory->Release();
		}
	}

	virtual void DestroyUI()
	{
		pluginState.posX = pluginWindow->GetPosX();
		pluginState.posY = pluginWindow->GetPosY();

		stateAssigned = true;

		pluginWindow->SetVisible(0);

		this->OnDestroyUI();

		pluginWindow->Release();

		pluginWindow = NULL;
	}


	// ============================== Override following methods to implement your plugin =============================
	

	// override this method to change window text
	virtual const wchar_t* GetPluginWindowText()
	{
		return this->GetPluginName();
	}

	// override this method to change plugin menu text
	virtual const wchar_t* GetPluginMenuText()
	{
		return this->GetPluginName();
	}

	// override this method to change plugin name
	virtual const wchar_t* GetPluginName()
	{
		return L"Unknown Plug-In";
	}

	// override this method to change plugin author
	virtual const wchar_t* GetPluginAuthor()
	{
		return L"Unknown Author";
	}

	// initialize/create plugin members. assign default values for plugin state.
	virtual int OnInitialize(int hostID, void* hostParam)
	{
		return 1;
	}

	// free the members which created at OnInitialize method.
	virtual void OnFree()
	{

	}

	// create controls on plugin window
	virtual void OnCreateUI(ARServiceFactory* stdServiceFactory, ARGUIService* guiService)
	{

	}

	// release controls
	virtual void OnDestroyUI()
	{

	}

	// called after updating the plugin state structure
	virtual void OnStateLoad()
	{

	}

	// called before saving plugin state structure
	virtual void OnStateSave()
	{

	}

	// apply plugin state to controls
	virtual void OnApplyPluginStateToUI()
	{

	}

	// start the process
	virtual void OnEnable()
	{

	}

	// stop the process
	virtual void OnDisable()
	{

	}
};
