
// Base classes which are binary compatibile with COM.
// (c) 2018-2021 CrownSoft

#ifndef _R_OBJBASE_H_
#define _R_OBJBASE_H_

#ifndef _CRT_SECURE_NO_WARNINGS
	#define _CRT_SECURE_NO_WARNINGS
#endif

#include <windows.h>

// types
#define R_BOOL int

// status values
#define R_OK 0
#define R_ERROR 1
#define R_NOINTERFACE 0x80004002L

// boolean values
#define R_TRUE 1
#define R_FALSE 0

#ifndef STDCALL
#define STDCALL __stdcall
#endif

typedef struct _RGUID {
	unsigned int Data1;
	unsigned short Data2;
	unsigned short Data3;
	unsigned char Data4[8];
} RGUID;

static_assert(sizeof(RGUID) == 16, "RGUID structure size is not correct");

int IsEqualRGUID(RGUID* rguid1, RGUID* rguid2)
{
	return !memcmp(rguid1, rguid2, sizeof(RGUID)); // if there are padding bytes then the static_assert above will fail!
}

class RUnknown
{
public:
	virtual long STDCALL QueryInterface(const RGUID& riid, void **ppvObject) = 0;
	virtual unsigned long STDCALL AddRef() = 0;
	virtual unsigned long STDCALL Release() = 0;
};

class CRUnknown : public RUnknown
{
protected:
	long refCount;

public:
	CRUnknown()
	{
		refCount = 1;
	}

	virtual long STDCALL QueryInterface(const RGUID& riid, void **ppvObject) override
	{
		return R_NOINTERFACE;
	}

	virtual unsigned long STDCALL AddRef() override
	{
		return InterlockedIncrement(&refCount);
	}

	virtual unsigned long STDCALL Release() override
	{
		long res = InterlockedDecrement(&refCount);
		if (res == 0)
			delete this;
		return res;
	}

	virtual ~CRUnknown(){}
};

#define IMPLEMENTS_QUERY_INTERFACE_METHOD virtual long STDCALL QueryInterface(const RGUID& riid, void **ppvObject) override { return R_NOINTERFACE; }
#define IMPLEMENTS_ADDREF_METHOD virtual unsigned long STDCALL AddRef() override { return CRUnknown::AddRef(); }
#define IMPLEMENTS_RELEASE_METHOD virtual unsigned long STDCALL Release() override { return CRUnknown::Release(); }

#define IMPLEMENTS_RUNKNOWN_METHODS IMPLEMENTS_QUERY_INTERFACE_METHOD \
	IMPLEMENTS_ADDREF_METHOD \
	IMPLEMENTS_RELEASE_METHOD

class RString : public RUnknown
{
public:
	virtual void STDCALL GetString(const wchar_t** ppText) = 0;
	virtual int STDCALL GetLength() = 0;
};

class CRString : public CRUnknown, public RString
{
protected:
	wchar_t* text;
	int length;

	~CRString()
	{
		if (text)
			::free(text);
	}

public:

	IMPLEMENTS_RUNKNOWN_METHODS

	CRString(const wchar_t* text, int len)
	{
		if (len)
		{
			this->text = (wchar_t*)::malloc((len + 1)*sizeof(wchar_t));
			length = len;
			::wcscpy(this->text, text);
		}
		else
		{
			if (text == NULL) // null string
			{
				this->text = NULL;
				length = 0;
			}
			else // empty string
			{
				this->text = (wchar_t*)::malloc(sizeof(wchar_t));
				this->text[0] = 0;
				length = len;
			}
		}
	}

	CRString(const wchar_t* text)
	{
		const int len = (int)::wcslen(text);
		if (len)
		{
			this->text = (wchar_t*)::malloc((len + 1)*sizeof(wchar_t));
			length = len;
			::wcscpy(this->text, text);
		}
		else
		{
			if (text == NULL) // null string
			{
				this->text = NULL;
				length = 0;
			}
			else // empty string
			{
				this->text = (wchar_t*)::malloc(sizeof(wchar_t));
				this->text[0] = 0;
				length = len;
			}
		}
	}

	CRString(const char* text)
	{
		const int len = (int)::strlen(text);
		if (len)
		{
			length = ::MultiByteToWideChar(CP_UTF8, 0, text, -1, 0, 0); // get char count with null character
			if (length)
			{
				this->text = (wchar_t *)::malloc(length * sizeof(wchar_t));
				if (::MultiByteToWideChar(CP_UTF8, 0, text, -1, this->text, length))
				{
					length--; // ignore null character
					return;
				}
				else
				{
					::free(this->text);
				}
			}

			this->text = NULL;
			length = 0;
		}
		else
		{
			if (text == NULL) // null string
			{
				this->text = NULL;
				length = 0;
			}
			else // empty string
			{
				this->text = (wchar_t*)::malloc(sizeof(wchar_t));
				this->text[0] = 0;
				length = len;
			}
		}
	}

	void STDCALL GetString(const wchar_t** ppText) override
	{
		*ppText = this->text;
	}

	int STDCALL GetLength() override
	{
		return length;
	}
};

inline RString* MakeRString(const char* text)
{
	return static_cast<RString*>(new CRString(text));
}

inline RString* MakeRString(const wchar_t* text)
{
	return static_cast<RString*>(new CRString(text));
}

inline RString* MakeRString(const wchar_t* text, int len)
{
	return static_cast<RString*>(new CRString(text, len));
}

class ScopedRString
{
private:
	RString* object;

	// Prevent heap allocation
	void* operator new(size_t);
	void* operator new[](size_t);
	void  operator delete(void*);
	void  operator delete[](void*);

public:
	ScopedRString()
	{
		object = NULL;
	}

	ScopedRString(const wchar_t* text)
	{
		object = MakeRString(text);
	}

	ScopedRString(const wchar_t* text, int len)
	{
		object = MakeRString(text, len);
	}

	ScopedRString(const char* text)
	{
		object = MakeRString(text);
	}

	ScopedRString(RString* object)
	{
		this->object = object;

		if (this->object)
			this->object->AddRef();
	}

	ScopedRString(ScopedRString& objectToTransferFrom)
	{
		object = objectToTransferFrom.object;

		if (object)
			object->AddRef();
	}

	bool IsNull()
	{
		return (object == NULL);
	}

	/**
		Removes the current RString object from this ScopedRString without releasing it.
		This will return the current object, and set the ScopedRString to a null pointer.
	*/
	RString* Detach()
	{
		RString* o = object;
		object = NULL;
		return o;
	}

	~ScopedRString()
	{
		if (object)
			object->Release();

		object = NULL;
	}

	inline RString** operator&() { return &object; }

	inline operator RString*() const { return object; }

	inline RString& operator*() const { return *object; }

	inline RString* operator->() const { return object; }

	inline operator const wchar_t* () const {
		if (object)
		{
			const wchar_t* text;
			object->GetString(&text);

			return text;
		}
		return NULL;
	}
};

#define SRSTR ScopedRString

#endif